@@ -47,6 +47,12 @@ pub enum QueryExecutionError {
     InvalidSlicing(Value),
     #[error("At least one of range start or end has to be specified")]
     UnboundedRange,
+    #[error("Invalid as base64")]
+    InvalidAsBase64(#[from] base64::DecodeError),
+    #[error("Invalid as a UTF-8-encoded byte array")]
+    InvalidUTF8Bytes(#[from] std::string::FromUtf8Error),
+    #[error("Invalid as a (c|t)sv entry: `{0:?}`")]
+    InvalidAsXSVEntry(Value),
     #[error("{0:?}")]
     UserDefinedError(String),
 }
